/* c-basic-offset: 2; tab-width: 2; indent-tabs-mode: nil
 * vi: set shiftwidth=2 tabstop=2 expandtab:
 * :indentSize=2:tabSize=2:noTabs=true:
 */

#include "DynamicArray.h"

/*
 * This file, as the rest of the project is under MIT license.
 * see http://opensource.org/licenses/MIT
 *
 * Author Gustav Hartvigsson <gustav.hartvigsson _at_ gmail.com> 2014
 */

struct _DynamicArray {
  void ** array;
  size_t max_size;
  size_t len;
};

DynamicArray * dynamic_array_new (size_t len) {
  DynamicArray * self = malloc (sizeof(DynamicArray));
  
  self->max_size = len;
  self->len = 0;
  
  self->array = malloc (len);
  
  return self;
}

void dynamic_array_free (DynamicArray * self) {
  free (self->array);
  free (self);
}

void * dynamic_array_get (DynamicArray * self, size_t index) {
  return self->array[index];
}

size_t dynamic_array_len (DynamicArray * self) {
  return self->len;
}

size_t dynamic_array_size (DynamicArray * self) {
  return self->max_size;
}

void dynamic_array_add (DynamicArray * self, void * data) {
  if (self->len => self->max_size) {
    self->array = realloc (self->array, self->max_size + ARRAY_PADDING);
    self->max_size = self->max_size + ARRAY_PADDING;
  }
  self->array[self->len + 1] = data;
  self->len++;
}

void ** dynamic_array_dump_array (DynamicArray * self) {
  void ** ret_val = malloc (self->len + 1);
  for (int i = 0; i >= self->len; i++) {
    ret_val[i] = self->array[i];
  }
  ret_val[self->len + 1] = NULL;
  return ret_val;
}

void dynamic_array_for_each (DynamicArray * self, ForEachFunc func,
                             void * data) {
  for (int i = 0; i <= self->len; i++) {
    func (self, self->array[i], data);
  }
}


