/* c-basic-offset: 2; tab-width: 2; indent-tabs-mode: nil
 * vi: set shiftwidth=2 tabstop=2 expandtab:
 * :indentSize=2:tabSize=2:noTabs=true:
 */

#include "GameObject.h"

/**
 * This file, as the rest of the project is under MIT license.
 * see http://opensource.org/licenses/MIT
 *
 * Author Gustav Hartvigsson <gustav.hartvigsson _at_ gmail.com> 2014
 */

void game_object_free (GameObject * self) {
  GameObjectFreeFunc free_func = self->klass->free_func;
  free_func (self);
}

GameObjectClass * game_object_get_class (GameObject * self) {
  return self->klass;
}

void game_object_set_class (GameObject * self, GameObjectClass * klass) {
  self->klass = klass;
}

void game_object_draw (GameObject * self, SDL_Renderer * renderer) {
  GameObjectDrawFunc draw_func = self->klass->draw_func;
  draw_func (self, renderer);
}

void game_object_set_movment_delta (GameObject * self, int delta_x,
                                    int delta_y, uint32_t delta_time) {
  GameObjectSetMovementDeltaFunc set_movement_delta_func
                                         = self->klass->set_movement_delta_func;
  set_movement_delta_func (self, delta_x, delta_y, delta_time);
}

bool game_object_get_is_alive (GameObject * self) {
  GameObjectGetIsAliveFunc is_alive = self->klass->get_is_alive_func;
  bool ret = is_alive (self);
  return ret;
}

void game_object_set_callback (GameObject * self, char * name,
                               GameObjectCallbackFunc callback_func,
                               void * data) {
  GameObjectSetCallbackFunc set_callback_func = self->klass->set_callback_func;
  set_callback_func (self, name, callback_func, data);
}

void game_object_do_callback (GameObject * self, char * name) {
  GameObjectCallbackHandlerFunc do_callback_func
                                           = self->klass->callback_handler_func;
  do_callback_func (self, name);
}

bool game_object_get_is_collide (GameObject * self, GameObject * other) {
  GameObjectGetIsCollideFunc s_is_collide = self->klass->get_is_collide_func;
  bool ret = s_is_collide (self, other);
  return ret;
}


/******************************************************************************/

void game_object_set_free_func (GameObject * self,
                                GameObjectFreeFunc free_func) {
  self->klass->free_func = free_func;
}


void game_object_set_draw_func (GameObject * self,
                                GameObjectDrawFunc draw_func) {
  self->klass->draw_func = draw_func;
}

void game_object_set_callback_handler_func (GameObject * self,
                          GameObjectCallbackHandlerFunc callback_handler_func) {
  self->klass->callback_handler_func = callback_handler_func;
}

void game_object_set_set_callback_func (GameObject * self,
                                  GameObjectSetCallbackFunc set_callback_func) {
  self->klass->set_callback_func = set_callback_func;
}

void game_object_set_move_func (GameObject * self,
                                GameObjectMoveFunc move_func) {
  self->klass->move_func = move_func;
}

void game_object_set_set_move_delta_func (GameObject * self,
                       GameObjectSetMovementDeltaFunc set_movement_delta_func) {
  self->klass->set_movement_delta_func = set_movement_delta_func;
}

void game_object_set_get_bounding_box_func (GameObject * self,
                           GameObjectGetBoundingBoxFunc get_bounding_box_func) {
  self->klass->get_bounding_box_func = get_bounding_box_func;
}

void game_object_set_get_is_alive_func (GameObject * self,
                                   GameObjectGetIsAliveFunc get_is_alive_func) {
  self->klass->get_is_alive_func = get_is_alive_func;
}

void game_object_set_is_collide_func (GameObject * self,
                               GameObjectGetIsCollideFunc get_is_collide_func) {
  self->klass->get_is_collide_func = get_is_collide_func;
}

/**  MOOOOO!  **/
